@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* Evolution calendar - Main page of the task editor dialog
  *
  * Copyright (C) 2001 Ximian, Inc.
@@ -78,13 +79,15 @@ struct _MeetingPagePrivate {
 	GtkWidget *add;
 	GtkWidget *remove;
 	GtkWidget *invite;
-	
+	GtkWidget *att_label;
+
 	/* ListView stuff */
 	EMeetingStore *model;
 	EMeetingListView *list_view;
 	gint row;
 	
 	/* For handling who the organizer is */
+	gboolean user_org;
 	gboolean existing;
 	
         gboolean updating;
@@ -316,12 +319,13 @@ clear_widgets (MeetingPage *mpage)
 	gtk_widget_hide (priv->existing_organizer_table);	
 
 	priv->existing = FALSE;
+	priv->user_org = TRUE;
 }
 
 static void
 sensitize_widgets (MeetingPage *mpage)
 {
-	gboolean read_only;
+	gboolean read_only, user_org;
 	MeetingPagePrivate *priv = mpage->priv;
 
 	if (!e_cal_is_read_only (COMP_EDITOR_PAGE (mpage)->client, &read_only, NULL))
@@ -329,9 +333,9 @@ sensitize_widgets (MeetingPage *mpage)
 
 	gtk_widget_set_sensitive (priv->organizer, !read_only);
 	gtk_widget_set_sensitive (priv->existing_organizer_btn, !read_only);
-	gtk_widget_set_sensitive (priv->add, !read_only);
-	gtk_widget_set_sensitive (priv->remove, !read_only);
-	gtk_widget_set_sensitive (priv->invite, !read_only);
+	gtk_widget_set_sensitive (priv->add, !read_only && priv->user_org);
+	gtk_widget_set_sensitive (priv->remove, !read_only && priv->user_org);
+	gtk_widget_set_sensitive (priv->invite, !read_only && priv->user_org);
 	gtk_widget_set_sensitive (GTK_WIDGET (priv->list_view), !read_only);
 }
 
@@ -379,6 +383,7 @@ meeting_page_fill_widgets (CompEditorPage *page, ECalComponent *comp)
 					    page->client,
 					    CAL_STATIC_CAPABILITY_ORGANIZER_NOT_EMAIL_ADDRESS))
 					gtk_widget_hide (priv->existing_organizer_btn);
+				priv->user_org = TRUE;
 			} else {
 				if (e_cal_get_static_capability (
 					    page->client,
@@ -387,6 +392,7 @@ meeting_page_fill_widgets (CompEditorPage *page, ECalComponent *comp)
 				gtk_widget_set_sensitive (priv->invite, FALSE);
 				gtk_widget_set_sensitive (priv->add, FALSE);
 				gtk_widget_set_sensitive (priv->remove, FALSE);
+				priv->user_org = FALSE;
 			}
 			
 			if (organizer.cn != NULL)
@@ -517,10 +523,14 @@ get_widgets (MeetingPage *mpage)
 	priv->add = GW ("add-attendee");
 	priv->remove = GW ("remove-attendee");
 	priv->invite = GW ("invite");
-	
+
+	/* Attendees Label */
+	priv->att_label = GW ("attendees-label");
+
 #undef GW
 
 	return (priv->list_box
+		&& priv->att_label
 		&& priv->invite
 		&& priv->add
 		&& priv->remove
@@ -580,6 +590,7 @@ change_clicked_cb (GtkWidget *widget, gpointer data)
 	comp_editor_page_notify_changed (COMP_EDITOR_PAGE (mpage));
 	
 	priv->existing = FALSE;
+	priv->user_org = TRUE;
 }
 
 static void
@@ -591,6 +602,32 @@ add_clicked_cb (GtkButton *btn, MeetingPage *mpage)
 	e_meeting_list_view_edit (mpage->priv->list_view, attendee);
 }
 
+static gboolean
+existing_attendee (EMeetingAttendee *ia, ECalComponent *comp) 
+{
+	GSList *attendees, *l;
+	const gchar *ia_address;
+	
+	ia_address = itip_strip_mailto (e_meeting_attendee_get_address (ia));
+	if (!ia_address)
+		return FALSE;
+	
+	e_cal_component_get_attendee_list (comp, &attendees);
+
+	for (l = attendees; l; l = l->next) {
+		ECalComponentAttendee *attendee = l->data;
+		const char *address;
+		
+		address = itip_strip_mailto (attendee->value);
+		if (address && !g_strcasecmp (ia_address, address))
+			return TRUE;
+	}
+	
+	e_cal_component_free_attendee_list (attendees);
+	
+	return FALSE;
+}
+
 static void
 remove_attendee (MeetingPage *mpage, EMeetingAttendee *ia) 
 {
@@ -599,8 +636,8 @@ remove_attendee (MeetingPage *mpage, EMeetingAttendee *ia)
 	
 	priv = mpage->priv;
 
-	/* If the user deletes the attendee explicitly, assume they no
-	   longer want the organizer showing up */
+	/* If the user deletes the organizer attendee explicitly,
+	   assume they no longer want the organizer showing up */
 	if (ia == priv->ia) {
 		g_object_unref (priv->ia);
 		priv->ia = NULL;
@@ -621,12 +658,15 @@ remove_attendee (MeetingPage *mpage, EMeetingAttendee *ia)
 	while (ia != NULL) {
 		EMeetingAttendee *ib = NULL;
 
-		g_object_ref (ia);
-		g_ptr_array_add (priv->deleted_attendees, ia);
-		e_meeting_store_remove_attendee (priv->model, ia);
-
+		if (existing_attendee (ia, priv->comp)) {
+			g_object_ref (ia);
+			g_ptr_array_add (priv->deleted_attendees, ia);
+		}
+		
 		if (e_meeting_attendee_get_delto (ia) != NULL)
 			ib = e_meeting_store_find_attendee (priv->model, e_meeting_attendee_get_delto (ia), NULL);
+		e_meeting_store_remove_attendee (priv->model, ia);
+
 		ia = ib;
 	}
 
@@ -892,8 +932,12 @@ meeting_page_construct (MeetingPage *mpage, EMeetingStore *ems,
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_IN);
 	gtk_widget_show (sw);
 	gtk_container_add (GTK_CONTAINER (sw), GTK_WIDGET (priv->list_view));
-	gtk_box_pack_start (GTK_BOX (priv->list_box), sw, TRUE, TRUE, 6);
+	gtk_box_pack_start (GTK_BOX (priv->list_box), sw, TRUE, TRUE, 0);
 	
+	/* Set the mnemonic widget for the Attendees label */
+	gtk_label_set_mnemonic_widget (GTK_LABEL (priv->att_label),
+				       priv->list_view);
+
 	/* Init the widget signals */
 	init_widgets (mpage);
 
