@@ -1,5 +1,6 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Copyright (C) 2000 Ximian Inc.
+ *  Copyright (C) 2000-2002 Ximian Inc.
  *
  *  Authors: Not Zed <notzed@lostzed.mmc.com.au>
  *           Jeffrey Stedfast <fejj@ximian.com>
@@ -19,35 +20,33 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
-#include <string.h>
-#include <glib.h>
-#include <gtk/gtkframe.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkmenuitem.h>
-#include <gtk/gtkoptionmenu.h>
-#include <gtk/gtkscrolledwindow.h>
-#include <gtk/gtktable.h>
-#include <libgnome/gnome-defs.h>
-#include <libgnome/gnome-i18n.h>
-#include <libgnomeui/gnome-stock.h>
-#include <gal/widgets/e-unicode.h>
+#endif
 
+#include <string.h>
+
+#include <gtk/gtk.h>
+#include <libgnome/gnome-i18n.h>
+
+#include "widgets/misc/e-error.h"
 #include "filter-rule.h"
-#include "filter-context.h"
+#include "rule-context.h"
+#include "filter-marshal.h"
 
 #define d(x)
 
 static int validate(FilterRule *);
+static int rule_eq(FilterRule *fr, FilterRule *cm);
 static xmlNodePtr xml_encode (FilterRule *);
 static int xml_decode (FilterRule *, xmlNodePtr, RuleContext *);
 static void build_code (FilterRule *, GString * out);
 static void rule_copy  (FilterRule *dest, FilterRule *src);
 static GtkWidget *get_widget (FilterRule * fr, struct _RuleContext *f);
 
-static void filter_rule_class_init (FilterRuleClass * class);
-static void filter_rule_init (FilterRule * gspaper);
-static void filter_rule_finalise (GtkObject * obj);
+static void filter_rule_class_init (FilterRuleClass *klass);
+static void filter_rule_init (FilterRule *fr);
+static void filter_rule_finalise (GObject *obj);
 
 #define _PRIVATE(x) (((FilterRule *)(x))->priv)
 
@@ -55,7 +54,7 @@ struct _FilterRulePrivate {
 	int frozen;
 };
 
-static GtkObjectClass *parent_class;
+static GObjectClass *parent_class = NULL;
 
 enum {
 	CHANGED,
@@ -64,77 +63,79 @@ enum {
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-guint
+GType
 filter_rule_get_type ()
 {
-	static guint type = 0;
+	static GType type = 0;
 	
 	if (!type) {
-		GtkTypeInfo type_info = {
-			"FilterRule",
-			sizeof (FilterRule),
+		static const GTypeInfo info = {
 			sizeof (FilterRuleClass),
-			(GtkClassInitFunc) filter_rule_class_init,
-			(GtkObjectInitFunc) filter_rule_init,
-			(GtkArgSetFunc) NULL,
-			(GtkArgGetFunc) NULL
+			NULL, /* base_class_init */
+			NULL, /* base_class_finalize */
+			(GClassInitFunc) filter_rule_class_init,
+			NULL, /* class_finalize */
+			NULL, /* class_data */
+			sizeof (FilterRule),
+			0,    /* n_preallocs */
+			(GInstanceInitFunc) filter_rule_init,
 		};
 		
-		type = gtk_type_unique (gtk_object_get_type(), &type_info);
+		type = g_type_register_static (G_TYPE_OBJECT, "FilterRule", &info, 0);
 	}
 	
 	return type;
 }
 
 static void
-filter_rule_class_init (FilterRuleClass * class)
+filter_rule_class_init (FilterRuleClass *klass)
 {
-	GtkObjectClass *object_class;
+	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	
-	object_class = (GtkObjectClass *) class;
-	parent_class = gtk_type_class (gtk_object_get_type());
+	parent_class = g_type_class_ref (G_TYPE_OBJECT);
 	
 	object_class->finalize = filter_rule_finalise;
 	
 	/* override methods */
-	class->validate = validate;
-	class->xml_encode = xml_encode;
-	class->xml_decode = xml_decode;
-	class->build_code = build_code;
-	class->copy = rule_copy;
-	class->get_widget = get_widget;
+	klass->validate = validate;
+	klass->eq = rule_eq;
+	klass->xml_encode = xml_encode;
+	klass->xml_decode = xml_decode;
+	klass->build_code = build_code;
+	klass->copy = rule_copy;
+	klass->get_widget = get_widget;
 	
 	/* signals */
 	signals[CHANGED] =
-		gtk_signal_new("changed",
-			       GTK_RUN_LAST,
-			       object_class->type,
-			       GTK_SIGNAL_OFFSET (FilterRuleClass, changed),
-			       gtk_marshal_NONE__NONE,
-			       GTK_TYPE_NONE, 0);
-	
-	gtk_object_class_add_signals (object_class, signals, LAST_SIGNAL);
+		g_signal_new ("changed",
+			      FILTER_TYPE_RULE,
+			      G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (FilterRuleClass, changed),
+			      NULL,
+			      NULL,
+			      filter_marshal_NONE__NONE,
+			      G_TYPE_NONE, 0);
 }
 
 static void
-filter_rule_init (FilterRule * o)
+filter_rule_init (FilterRule *fr)
 {
-	o->priv = g_malloc0 (sizeof (*o->priv));
+	fr->priv = g_malloc0 (sizeof (*fr->priv));
 }
 
 static void
-filter_rule_finalise (GtkObject * obj)
+filter_rule_finalise (GObject *obj)
 {
-	FilterRule *o = (FilterRule *) obj;
-
-	g_free (o->name);
-	g_free (o->source);
-	g_list_foreach(o->parts, (GFunc)gtk_object_unref, NULL);
-	g_list_free(o->parts);
-
-	g_free(o->priv);
+	FilterRule *fr = (FilterRule *) obj;
 	
-	((GtkObjectClass *) (parent_class))->finalize(obj);
+	g_free (fr->name);
+	g_free (fr->source);
+	g_list_foreach (fr->parts, (GFunc)g_object_unref, NULL);
+	g_list_free (fr->parts);
+	
+	g_free (fr->priv);
+	
+	G_OBJECT_CLASS (parent_class)->finalize (obj);
 }
 
 /**
@@ -147,9 +148,7 @@ filter_rule_finalise (GtkObject * obj)
 FilterRule *
 filter_rule_new ()
 {
-	FilterRule *o = (FilterRule *) gtk_type_new (filter_rule_get_type());
-	
-	return o;
+	return (FilterRule *) g_object_new (FILTER_TYPE_RULE, NULL, NULL);
 }
 
 FilterRule *
@@ -159,7 +158,7 @@ filter_rule_clone (FilterRule *base)
 	
 	g_assert (IS_FILTER_RULE (base));
 	
-	rule = gtk_type_new (GTK_OBJECT (base)->klass->type);
+	rule = g_object_new (G_OBJECT_TYPE (base), NULL, NULL);
 	filter_rule_copy (rule, base);
 	
 	return rule;
@@ -169,30 +168,30 @@ void
 filter_rule_set_name (FilterRule *fr, const char *name)
 {
 	g_assert (IS_FILTER_RULE (fr));
-
-	if ((fr->name && name && strcmp(fr->name, name) == 0)
+	
+	if ((fr->name && name && strcmp (fr->name, name) == 0)
 	    || (fr->name == NULL && name == NULL))
 		return;
 	
 	g_free (fr->name);
 	fr->name = g_strdup (name);
-
-	filter_rule_emit_changed(fr);
+	
+	filter_rule_emit_changed (fr);
 }
 
 void
 filter_rule_set_source (FilterRule *fr, const char *source)
 {
 	g_assert (IS_FILTER_RULE (fr));
-
-	if ((fr->source && source && strcmp(fr->source, source) == 0)
+	
+	if ((fr->source && source && strcmp (fr->source, source) == 0)
 	    || (fr->source == NULL && source == NULL))
 		return;
 	
 	g_free (fr->source);
 	fr->source = g_strdup (source);
-
-	filter_rule_emit_changed(fr);
+	
+	filter_rule_emit_changed (fr);
 }
 
 int
@@ -200,7 +199,7 @@ filter_rule_validate (FilterRule *fr)
 {
 	g_assert (IS_FILTER_RULE (fr));
 	
-	return ((FilterRuleClass *) ((GtkObject *) fr)->klass)->validate (fr);
+	return FILTER_RULE_GET_CLASS (fr)->validate (fr);
 }
 
 static int
@@ -209,22 +208,70 @@ validate (FilterRule *fr)
 	int valid = TRUE;
 	GList *parts;
 	
+	if (!fr->name || !*fr->name) {
+		/* FIXME: FilterElement should probably have a
+                   GtkWidget member pointing to the value gotten with
+                   ::get_widget() so that we can get the parent window
+                   here. */
+		e_error_run(NULL, "filter:no-name", NULL);
+		
+		return FALSE;
+	}
+	
 	/* validate rule parts */
 	parts = fr->parts;
 	while (parts && valid) {
-		valid = filter_part_validate ((FilterPart *)parts->data);
+		valid = filter_part_validate ((FilterPart *) parts->data);
 		parts = parts->next;
 	}
 	
 	return valid;
 }
 
+int
+filter_rule_eq (FilterRule *fr, FilterRule *cm)
+{
+	g_assert (IS_FILTER_RULE (fr));
+	g_assert (IS_FILTER_RULE (cm));
+	
+	return (FILTER_RULE_GET_CLASS (fr) == FILTER_RULE_GET_CLASS (cm))
+		&& FILTER_RULE_GET_CLASS (fr)->eq (fr, cm);
+}
+
+static int
+list_eq(GList *al, GList *bl)
+{
+	int truth = TRUE;
+	
+	while (truth && al && bl) {
+		FilterPart *a = al->data, *b = bl->data;
+		
+		truth = filter_part_eq (a, b);
+		al = al->next;
+		bl = bl->next;
+	}
+	
+	return truth && al == NULL && bl == NULL;
+}
+
+static int
+rule_eq (FilterRule *fr, FilterRule *cm)
+{
+	return fr->grouping == cm->grouping
+		&& fr->threading == fr->threading
+		&& ((fr->name && cm->name && strcmp (fr->name, cm->name) == 0)
+		     || (fr->name == NULL && cm->name == NULL))
+		&& ((fr->source && cm->source && strcmp (fr->source, cm->source) == 0)
+		     || (fr->source == NULL && cm->source == NULL) )
+		&& list_eq (fr->parts, cm->parts);
+}
+
 xmlNodePtr
 filter_rule_xml_encode (FilterRule *fr)
 {
 	g_assert (IS_FILTER_RULE (fr));
 	
-	return ((FilterRuleClass *) ((GtkObject *) fr)->klass)->xml_encode (fr);
+	return FILTER_RULE_GET_CLASS (fr)->xml_encode (fr);
 }
 
 static xmlNodePtr
@@ -242,7 +289,21 @@ xml_encode (FilterRule *fr)
 		xmlSetProp (node, "grouping", "any");
 		break;
 	}
-	
+
+	switch (fr->threading) {
+	case FILTER_THREAD_NONE:
+		break;
+	case FILTER_THREAD_ALL:
+		xmlSetProp(node, "threading", "all");
+		break;
+	case FILTER_THREAD_REPLIES:
+		xmlSetProp(node, "threading", "replies");
+		break;
+	case FILTER_THREAD_REPLIES_PARENTS:
+		xmlSetProp(node, "threading", "replies_parents");
+		break;
+	}
+
 	if (fr->source) {
 		xmlSetProp (node, "source", fr->source);
 	} else {
@@ -251,12 +312,8 @@ xml_encode (FilterRule *fr)
 	}
 	
 	if (fr->name) {
-		char *encstr;
-		
 		work = xmlNewNode (NULL, "title");
-		encstr = e_utf8_xml1_encode (fr->name);
-		xmlNodeSetContent (work, encstr);
-		g_free (encstr);
+		xmlNodeSetContent (work, fr->name);
 		xmlAddChild (node, work);
 	}
 	
@@ -266,7 +323,7 @@ xml_encode (FilterRule *fr)
 	while (l) {
 		work = filter_part_xml_encode ((FilterPart *) l->data);
 		xmlAddChild (set, work);
-		l = g_list_next (l);
+		l = l->next;
 	}
 	
 	return node;
@@ -279,7 +336,7 @@ load_set (xmlNodePtr node, FilterRule *fr, RuleContext *f)
 	char *rulename;
 	FilterPart *part;
 	
-	work = node->childs;
+	work = node->children;
 	while (work) {
 		if (!strcmp (work->name, "part")) {
 			rulename = xmlGetProp (work, "name");
@@ -292,8 +349,8 @@ load_set (xmlNodePtr node, FilterRule *fr, RuleContext *f)
 				g_warning ("cannot find rule part '%s'\n", rulename);
 			}
 			xmlFree (rulename);
-		} else {
-			g_warning ("Unknwon xml node in part: %s", work->name);
+		} else if (work->type == XML_ELEMENT_NODE) {
+			g_warning ("Unknown xml node in part: %s", work->name);
 		}
 		work = work->next;
 	}
@@ -303,17 +360,17 @@ int
 filter_rule_xml_decode (FilterRule *fr, xmlNodePtr node, RuleContext *f)
 {
 	int res;
-
+	
 	g_assert (IS_FILTER_RULE (fr));
 	g_assert (IS_RULE_CONTEXT (f));
 	g_assert (node != NULL);
-
+	
 	fr->priv->frozen++;	
-	res = ((FilterRuleClass *) ((GtkObject *) fr)->klass)->xml_decode (fr, node, f);
+	res = FILTER_RULE_GET_CLASS (fr)->xml_decode (fr, node, f);
 	fr->priv->frozen--;
-
-	filter_rule_emit_changed(fr);
-
+	
+	filter_rule_emit_changed (fr);
+	
 	return res;
 }
 
@@ -335,6 +392,18 @@ xml_decode (FilterRule *fr, xmlNodePtr node, RuleContext *f)
 	else
 		fr->grouping = FILTER_GROUP_ALL;
 	xmlFree (grouping);
+
+	fr->threading = FILTER_THREAD_NONE;
+	if (f->flags & RULE_CONTEXT_THREADING
+	    && (grouping = xmlGetProp (node, "threading"))) {
+		if (!strcmp(grouping, "all"))
+			fr->threading = FILTER_THREAD_ALL;
+		else if (!strcmp(grouping, "replies"))
+			fr->threading = FILTER_THREAD_REPLIES;
+		else if (!strcmp(grouping, "replies_parents"))
+			fr->threading = FILTER_THREAD_REPLIES_PARENTS;
+		xmlFree (grouping);
+	}
 	
 	g_free (fr->source);
 	source = xmlGetProp (node, "source");
@@ -346,16 +415,16 @@ xml_decode (FilterRule *fr, xmlNodePtr node, RuleContext *f)
 		fr->source = g_strdup ("incoming");
 	}
 	
-	work = node->childs;
+	work = node->children;
 	while (work) {
 		if (!strcmp (work->name, "partset")) {
 			load_set (work, fr, f);
 		} else if (!strcmp (work->name, "title") || !strcmp (work->name, "_title")) {
 			if (!fr->name) {
 				char *str, *decstr;
-
+				
 				str = xmlNodeGetContent (work);
-				decstr = e_utf8_xml1_decode (str);
+				decstr = g_strdup (str);
 				if (str)
 					xmlFree (str);
 				fr->name = decstr;
@@ -379,18 +448,19 @@ rule_copy (FilterRule *dest, FilterRule *src)
 	dest->source = g_strdup (src->source);
 	
 	dest->grouping = src->grouping;
-	
+	dest->threading = src->threading;
+
 	if (dest->parts) {
-		g_list_foreach (dest->parts, (GFunc) gtk_object_unref, NULL);
+		g_list_foreach (dest->parts, (GFunc) g_object_unref, NULL);
 		g_list_free (dest->parts);
 		dest->parts = NULL;
 	}
 	
 	node = src->parts;
 	while (node) {
-		FilterPart *part = node->data;
-		
-		gtk_object_ref (GTK_OBJECT (part));
+		FilterPart *part;
+
+		part = filter_part_clone (node->data);
 		dest->parts = g_list_append (dest->parts, part);
 		node = node->next;
 	}
@@ -402,9 +472,9 @@ filter_rule_copy (FilterRule *dest, FilterRule *src)
 	g_assert (IS_FILTER_RULE (dest));
 	g_assert (IS_FILTER_RULE (src));
 	
-	((FilterRuleClass *) ((GtkObject *) dest)->klass)->copy (dest, src);
-
-	filter_rule_emit_changed(dest);
+	FILTER_RULE_GET_CLASS (dest)->copy (dest, src);
+	
+	filter_rule_emit_changed (dest);
 }
 
 void
@@ -414,8 +484,8 @@ filter_rule_add_part (FilterRule *fr, FilterPart *fp)
 	g_assert (IS_FILTER_PART (fp));
 	
 	fr->parts = g_list_append (fr->parts, fp);
-
-	filter_rule_emit_changed(fr);
+	
+	filter_rule_emit_changed (fr);
 }
 
 void
@@ -425,8 +495,8 @@ filter_rule_remove_part (FilterRule *fr, FilterPart *fp)
 	g_assert (IS_FILTER_PART (fp));
 	
 	fr->parts = g_list_remove (fr->parts, fp);
-
-	filter_rule_emit_changed(fr);
+	
+	filter_rule_emit_changed (fr);
 }
 
 void
@@ -444,8 +514,8 @@ filter_rule_replace_part (FilterRule *fr, FilterPart *fp, FilterPart *new)
 	} else {
 		fr->parts = g_list_append (fr->parts, new);
 	}
-
-	filter_rule_emit_changed(fr);
+	
+	filter_rule_emit_changed (fr);
 }
 
 void
@@ -454,7 +524,7 @@ filter_rule_build_code (FilterRule *fr, GString *out)
 	g_assert (IS_FILTER_RULE (fr));
 	g_assert (out != NULL);
 	
-	((FilterRuleClass *) ((GtkObject *) fr)->klass)->build_code (fr, out);
+	FILTER_RULE_GET_CLASS (fr)->build_code (fr, out);
 	
 	d(printf ("build_code: [%s](%d)", out->str, out->len));
 }
@@ -463,14 +533,28 @@ void
 filter_rule_emit_changed(FilterRule *fr)
 {
 	g_assert (IS_FILTER_RULE (fr));
-
+	
 	if (fr->priv->frozen == 0)
-		gtk_signal_emit((GtkObject *)fr, signals[CHANGED]);
+		g_signal_emit (fr, signals[CHANGED], 0);
 }
 
 static void
 build_code (FilterRule *fr, GString *out)
 {
+	switch (fr->threading) {
+	case FILTER_THREAD_NONE:
+		break;
+	case FILTER_THREAD_ALL:
+		g_string_append(out, " (match-threads \"all\" ");
+		break;
+	case FILTER_THREAD_REPLIES:
+		g_string_append(out, " (match-threads \"replies\" ");
+		break;
+	case FILTER_THREAD_REPLIES_PARENTS:
+		g_string_append(out, " (match-threads \"replies_parents\" ");
+		break;
+	}
+
 	switch (fr->grouping) {
 	case FILTER_GROUP_ALL:
 		g_string_append (out, " (and\n  ");
@@ -484,18 +568,21 @@ build_code (FilterRule *fr, GString *out)
 	
 	filter_part_build_code_list (fr->parts, out);
 	g_string_append (out, ")\n");
+
+	if (fr->threading != FILTER_THREAD_NONE)
+		g_string_append (out, ")\n");
 }
 
 static void
-match_all (GtkWidget *widget, FilterRule *fr)
+fr_grouping_changed(GtkWidget *w, FilterRule *fr)
 {
-	fr->grouping = FILTER_GROUP_ALL;
+	fr->grouping = gtk_option_menu_get_history((GtkOptionMenu *)w);
 }
 
 static void
-match_any (GtkWidget *widget, FilterRule *fr)
+fr_threading_changed(GtkWidget *w, FilterRule *fr)
 {
-	fr->grouping = FILTER_GROUP_ANY;
+	fr->threading = gtk_option_menu_get_history((GtkOptionMenu *)w);
 }
 
 struct _part_data {
@@ -508,7 +595,7 @@ struct _part_data {
 static void
 option_activate (GtkMenuItem *item, struct _part_data *data)
 {
-	FilterPart *part = gtk_object_get_data (GTK_OBJECT (item), "part");
+	FilterPart *part = g_object_get_data ((GObject *) item, "part");
 	FilterPart *newpart;
 	
 	/* dont update if we haven't changed */
@@ -521,14 +608,15 @@ option_activate (GtkMenuItem *item, struct _part_data *data)
 		gtk_container_remove (GTK_CONTAINER (data->container), data->partwidget);
 	
 	newpart = filter_part_clone (part);
+	filter_part_copy_values (newpart, data->part);
 	filter_rule_replace_part (data->fr, data->part, newpart);
-	gtk_object_unref (GTK_OBJECT (data->part));
+	g_object_unref (data->part);
 	data->part = newpart;
 	data->partwidget = filter_part_get_widget (newpart);
 	if (data->partwidget)
 		gtk_box_pack_start (GTK_BOX (data->container), data->partwidget, FALSE, FALSE, 0);
 	
-	gtk_object_set_data (GTK_OBJECT (data->container), "part", newpart);
+	g_object_set_data ((GObject *) data->container, "part", newpart);
 }
 
 static GtkWidget *
@@ -550,7 +638,7 @@ get_rule_part_widget (RuleContext *f, FilterPart *newpart, FilterRule *fr)
 	
 	hbox = gtk_hbox_new (FALSE, 0);
 	/* only set to automatically clean up the memory */
-	gtk_object_set_data_full (GTK_OBJECT (hbox), "data", data, g_free);
+	g_object_set_data_full ((GObject *) hbox, "data", data, g_free);
 	
 	p = filter_part_get_widget (newpart);
 	
@@ -561,13 +649,13 @@ get_rule_part_widget (RuleContext *f, FilterPart *newpart, FilterRule *fr)
 	/* sigh, this is a little ugly */
 	while ((part = rule_context_next_part (f, part))) {
 		item = gtk_menu_item_new_with_label (_(part->title));
-		gtk_object_set_data (GTK_OBJECT (item), "part", part);
-		gtk_signal_connect (GTK_OBJECT (item), "activate", option_activate, data);
-		gtk_menu_append (GTK_MENU (menu), item);
+		g_object_set_data ((GObject *) item, "part", part);
+		g_signal_connect (item, "activate", G_CALLBACK (option_activate), data);
+		gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 		gtk_widget_show (item);
-		if (!strcmp (newpart->title, part->title)) {
+		if (!strcmp (newpart->title, part->title))
 			current = index;
-		}
+		
 		index++;
 	}
 	
@@ -577,9 +665,9 @@ get_rule_part_widget (RuleContext *f, FilterPart *newpart, FilterRule *fr)
 	gtk_widget_show (omenu);
 	
 	gtk_box_pack_start (GTK_BOX (hbox), omenu, FALSE, FALSE, 0);
-	if (p) {
+	if (p)
 		gtk_box_pack_start (GTK_BOX (hbox), p, FALSE, FALSE, 0);
-	}
+	
 	gtk_widget_show_all (hbox);
 	
 	return hbox;
@@ -600,12 +688,12 @@ less_parts (GtkWidget *button, struct _rule_data *data)
 	if (g_list_length (data->fr->parts) < 2)
 		return;
 	
-	rule = gtk_object_get_data (GTK_OBJECT (button), "rule");
-	part = gtk_object_get_data (GTK_OBJECT (rule), "part");
+	rule = g_object_get_data ((GObject *) button, "rule");
+	part = g_object_get_data ((GObject *) rule, "part");
 	
 	/* remove the part from the list */
 	filter_rule_remove_part (data->fr, part);
-	gtk_object_unref (GTK_OBJECT (part));
+	g_object_unref (part);
 	
 	/* and from the display */
 	gtk_container_remove (GTK_CONTAINER (data->parts), rule);
@@ -616,19 +704,18 @@ static void
 attach_rule (GtkWidget *rule, struct _rule_data *data, FilterPart *part, int row)
 {
 	GtkWidget *remove;
-	GtkWidget *pixmap;
 	
 	gtk_table_attach (GTK_TABLE (data->parts), rule, 0, 1, row, row + 1,
 			  GTK_EXPAND | GTK_FILL, 0, 0, 0);
 	
-	pixmap = gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_REMOVE);
-	remove = gnome_pixmap_button (pixmap, _("Remove"));
-	gtk_object_set_data (GTK_OBJECT (remove), "rule", rule);
-	gtk_object_set_data (GTK_OBJECT (rule), "part", part);
+	remove = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
+	g_object_set_data ((GObject *) remove, "rule", rule);
+	g_object_set_data ((GObject *) rule, "part", part);
 	/*gtk_button_set_relief (GTK_BUTTON (remove), GTK_RELIEF_NONE);*/
-	gtk_signal_connect (GTK_OBJECT (remove), "clicked", less_parts, data);
+	g_signal_connect (remove, "clicked", G_CALLBACK (less_parts), data);
 	gtk_table_attach (GTK_TABLE (data->parts), remove, 1, 2, row, row + 1,
 			  0, 0, 0, 0);
+	
 	gtk_widget_show (remove);
 }
 
@@ -668,13 +755,13 @@ static void
 name_changed (GtkEntry *entry, FilterRule *fr)
 {
 	g_free (fr->name);
-	fr->name = e_utf8_gtk_entry_get_text (entry);
+	fr->name = g_strdup (gtk_entry_get_text (entry));
 }
 
 GtkWidget *
-filter_rule_get_widget (FilterRule *fr, struct _RuleContext *f)
+filter_rule_get_widget (FilterRule *fr, RuleContext *rc)
 {
-	return ((FilterRuleClass *) ((GtkObject *) fr)->klass)->get_widget(fr, f);
+	return FILTER_RULE_GET_CLASS (fr)->get_widget (fr, rc);
 }
 
 static void
@@ -686,26 +773,21 @@ grab_focus (GtkWidget *entry, gpointer data)
 static GtkWidget *
 get_widget (FilterRule *fr, struct _RuleContext *f)
 {
-	GtkWidget *vbox, *parts, *inframe;
-	GtkWidget *hbox;
-	GtkWidget *add, *pixmap;
-	GtkWidget *w;
-	GtkWidget *menu, *item, *omenu;
-	GtkWidget *frame;
-	GtkWidget *name;
-	GtkWidget *label;
+	GtkWidget *hbox, *vbox, *parts, *inframe;
+	GtkWidget *add, *label, *name, *w;
+	GtkWidget *omenu, *menu, *item;
 	GtkWidget *scrolledwindow;
 	GtkObject *hadj, *vadj;
 	GList *l;
 	FilterPart *part;
 	struct _rule_data *data;
-	gint rows, i = 0;
+	int rows, i;
 	
 	/* this stuff should probably be a table, but the
 	   rule parts need to be a vbox */
-	vbox = gtk_vbox_new (FALSE, 3);
-	
-	label = gtk_label_new (_("Rule name: "));
+	vbox = gtk_vbox_new (FALSE, 6);
+
+	label = gtk_label_new (_("Rule name:"));
 	name = gtk_entry_new ();
 	
 	if (!fr->name) {
@@ -714,21 +796,40 @@ get_widget (FilterRule *fr, struct _RuleContext *f)
 		/* FIXME: do we want the following code in the future? */
 		/*gtk_editable_select_region (GTK_EDITABLE (name), 0, -1);*/
 	} else {
-		e_utf8_gtk_entry_set_text (GTK_ENTRY (name), fr->name);
+		gtk_entry_set_text (GTK_ENTRY (name), fr->name);
 	}
 	
 	/* evil kludgy hack because gtk sucks */
-	gtk_signal_connect (GTK_OBJECT (name), "realize", grab_focus, name);
+	g_signal_connect (name, "realize", G_CALLBACK (grab_focus), name);
 	
-	hbox = gtk_hbox_new (FALSE, 3);
+	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (hbox), name, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
-	gtk_signal_connect (GTK_OBJECT (name), "changed", name_changed, fr);
-	
-	frame = gtk_frame_new (_("If"));
-	inframe = gtk_vbox_new (FALSE, 3);
-	gtk_container_add (GTK_CONTAINER (frame), inframe);
+	g_signal_connect (name, "changed", G_CALLBACK (name_changed), fr);
+	gtk_widget_show (label);
+	gtk_widget_show (hbox);
+
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show (hbox);
+
+	label = gtk_label_new (_("<b>If</b>"));
+	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_widget_show (label);
+
+	hbox = gtk_hbox_new (FALSE, 12);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);
+	gtk_widget_show (hbox);
+
+	label = gtk_label_new ("");
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	gtk_widget_show (label);
+
+	inframe = gtk_vbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (hbox), inframe, TRUE, TRUE, 0);
 	
 	/* this is the parts table, it should probably be inside a scrolling list */
 	rows = g_list_length (fr->parts);
@@ -741,39 +842,64 @@ get_widget (FilterRule *fr, struct _RuleContext *f)
 	data->parts = parts;
 	
 	/* only set to automatically clean up the memory */
-	gtk_object_set_data_full (GTK_OBJECT (vbox), "data", data, g_free);
+	g_object_set_data_full ((GObject *) vbox, "data", data, g_free);
 	
 	hbox = gtk_hbox_new (FALSE, 3);
-	label = gtk_label_new (_("Execute actions"));
+
+	add = gtk_button_new_from_stock (GTK_STOCK_ADD);
+	g_signal_connect (add, "clicked", G_CALLBACK (more_parts), data);
+	gtk_box_pack_start (GTK_BOX (hbox), add, FALSE, FALSE, 0);
+
+	if (f->flags & RULE_CONTEXT_GROUPING) {
+		const char *thread_types[] = { N_("if all criteria are met"), N_("if any criteria are met") };
+
+		label = gtk_label_new (_("Execute actions"));
+		menu = gtk_menu_new ();
 	
-	menu = gtk_menu_new ();
+		for (i=0;i<2;i++) {
+			item = gtk_menu_item_new_with_label(_(thread_types[i]));
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+			gtk_widget_show (item);
+		}
 	
-	item = gtk_menu_item_new_with_label (_("if all criteria are met"));
-	gtk_signal_connect (GTK_OBJECT (item), "activate", match_all, fr);
-	gtk_menu_append (GTK_MENU (menu), item);
-	gtk_widget_show (item);
+		omenu = gtk_option_menu_new ();
+		gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
+		gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), fr->grouping);
+		gtk_widget_show (omenu);
 	
-	item = gtk_menu_item_new_with_label (_("if any criteria are met"));
-	gtk_signal_connect (GTK_OBJECT (item), "activate", match_any, fr);
-	gtk_menu_append (GTK_MENU (menu), item);
-	gtk_widget_show (item);
+		gtk_box_pack_end (GTK_BOX (hbox), omenu, FALSE, FALSE, 0);
+		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+		g_signal_connect(omenu, "changed", G_CALLBACK(fr_grouping_changed), fr);
+	}
+
+	if (f->flags & RULE_CONTEXT_THREADING) {
+		const char *thread_types[] = { N_("None"), N_("All related"), N_("Replies"), N_("Replies and parents") };
+
+		label = gtk_label_new (_("Include threads"));
+		menu = gtk_menu_new ();
 	
-	omenu = gtk_option_menu_new ();
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), fr->grouping == FILTER_GROUP_ALL ? 0 : 1);
-	gtk_widget_show (omenu);
+		for (i=0;i<4;i++) {
+			item = gtk_menu_item_new_with_label(_(thread_types[i]));
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+			gtk_widget_show (item);
+		}
 	
-	pixmap = gnome_stock_new_with_icon (GNOME_STOCK_PIXMAP_ADD);
-	add = gnome_pixmap_button (pixmap, _("Add criterion"));
-	gtk_signal_connect (GTK_OBJECT (add), "clicked", more_parts, data);
-	gtk_box_pack_start (GTK_BOX (hbox), add, FALSE, FALSE, 3);
+		omenu = gtk_option_menu_new ();
+		gtk_option_menu_set_menu (GTK_OPTION_MENU (omenu), menu);
+		gtk_option_menu_set_history (GTK_OPTION_MENU (omenu), fr->threading);
+		gtk_widget_show (omenu);
 	
-	gtk_box_pack_end (GTK_BOX (hbox), omenu, FALSE, FALSE, 0);
-	gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+		gtk_box_pack_end (GTK_BOX (hbox), omenu, FALSE, FALSE, 0);
+		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+		g_signal_connect(omenu, "changed", G_CALLBACK(fr_threading_changed), fr);
+	}
 	
-	gtk_box_pack_start (GTK_BOX (inframe), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (inframe), hbox, FALSE, FALSE, 3);
 	
 	l = fr->parts;
+	i = 0;
 	while (l) {
 		part = l->data;
 		d(printf ("adding rule %s\n", part->title));
@@ -791,11 +917,7 @@ get_widget (FilterRule *fr, struct _RuleContext *f)
 	
 	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolledwindow), parts);
 	
-	gtk_box_pack_start (GTK_BOX (inframe), scrolledwindow, TRUE, TRUE, 0);
-	
-	/*gtk_box_pack_start (GTK_BOX (inframe), parts, FALSE, FALSE, 3); */
-	
-	gtk_box_pack_start (GTK_BOX (vbox), frame, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (inframe), scrolledwindow, TRUE, TRUE, 3);
 	
 	gtk_widget_show_all (vbox);
 	
@@ -803,7 +925,7 @@ get_widget (FilterRule *fr, struct _RuleContext *f)
 }
 
 FilterRule *
-filter_rule_next_list (GList * l, FilterRule * last, const char *source)
+filter_rule_next_list (GList *l, FilterRule *last, const char *source)
 {
 	GList *node = l;
 	
@@ -812,7 +934,7 @@ filter_rule_next_list (GList * l, FilterRule * last, const char *source)
 		if (node == NULL)
 			node = l;
 		else
-			node = g_list_next (node);
+			node = node->next;
 	}
 	
 	if (source) {
@@ -821,7 +943,7 @@ filter_rule_next_list (GList * l, FilterRule * last, const char *source)
 			
 			if (rule->source && strcmp (rule->source, source) == 0)
 				break;
-			node = g_list_next (node);
+			node = node->next;
 		}
 	}
 	
@@ -840,7 +962,7 @@ filter_rule_find_list (GList * l, const char *name, const char *source)
 		if (strcmp (rule->name, name) == 0)
 			if (source == NULL || (rule->source != NULL && strcmp (rule->source, source) == 0))
 				return rule;
-		l = g_list_next (l);
+		l = l->next;
 	}
 	
 	return NULL;
